##==============================================================================
## calcindex.R :: tools for calculating various indexes
##------------------------------------------------------------------------------
## Tae-Hoon Chung (hoontaechung@gmail.com)
##
##     Copyright (c) 2009, 2010, Tae-Hoon Chung. All rights reserved.
##
##     Redistribution and use in source and binary forms, with or without
##     modification, are permitted provided that the following conditions are
##     met:
##
##         - Redistributions of source code must retain the above copyright
##         notice, this list of conditions and the following disclaimer.
##
##         - Redistributions in binary form must reproduce the above
##         copyright notice, this list of conditions and the following
##         disclaimer in the documentation and/or other materials provided
##         with the distribution.
##
##         - Neither the name of the Tae-Hoon Chung nor the names of its
##         contributors may be used to endorse or promote products derived
##         from this software without specific prior written permission.
##
##     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
##     "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
##     LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
##     A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
##     OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
##     SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
##     LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
##     DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
##     THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
##     (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
##     OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
##
##==============================================================================

ALG <- c("median", "mean")
VALUE.TYPE <- c("intensity", "logintensity", "logratio")

fGetDB <- function(name, dir, new=TRUE) {
  drv <- dbDriver("SQLite")
  if (!missing(dir)) {
    dd <- tools::file_path_as_absolute(dir)
    dbname <- file.path(dd, name)
  } else {
    dbname <- name
  }
  if (!new) {
    if (!file.exists(dbname)) stop(paste("Could not open", name))
  } else {
    if (file.exists(dbname)) stop(paste("Existing DB", name))
  }
  dbConnect(drv, dbname=dbname)
}

fTemplate <- function(txt, ...) {
  ## require(gdata)  ## for trim()
  li <- list(...)
  li2 <- unlist(strsplit(txt, "%", fixed=T))
  for (n in 1:length(li2)) {
    x <- trim(li2[n])
    if (x %in% names(li)) {
      li2[n] <- li[[x]]
    }
  }
  paste(unlist(li2), collapse="")
}

fReleaseDB <- function(con) {
  dbDisconnect(con)
}


#################################################################
## utility function to generate file paths :: concatenating strings
## without gaps.
## fpack("AAA", "BBB", "CCC") -> "AAABBBCCC"
#################################################################
fpack <- function(...) {
  li <- list(...)
  if (length(li) == 1) {
    paste(..., collapse="")
  } else {
    paste(..., sep="")
  }
}

## ... simple utility function to categorize input vector into discrete levels
fcategorize <- function(x, nlev, by=c("equal.lev", "equal.quantile", "fixed.lev", "fixed.quantile", "fixed.range")) {
  by = pmatch(by, c("equal.lev", "equal.quantile", "fixed.lev", "fixed.quantile", "fixed.range"))
  if (by == 1) {
    lev <- seq(min(x, na.rm=T), max(x, na.rm=T), len=nlev+1)
    lev[which.min(lev)] <- min(x, na.rm=T)-1
    lev[which.max(lev)] <- max(x, na.rm=T)+1
    cut(x, lev, labels=F)
  } else if (by == 2) {
    stp <- seq(0, 1, len=nlev+1)
    lev <- c(min(x, na.rm=T)-1, quantile(x, prob=stp[-c(1,nlev+1)]), max(x, na.rm=T)+1)
    cut(x, lev, labels=F)
  } else if (by == 3) {
    m <- min(nlev); M <- max(nlev)
    if (any(x <= m, na.rm=T)) {
      if (all(x < M, na.rm=T)){
        lev <- c(min(x, na.rm=T)-1, sort(nlev))
      } else {
        lev <- c(min(x, na.rm=T)-1, sort(nlev), max(x, na.rm=T)+1)
      }
    } else {
      if (all(x < M, na.rm=T)) {
        lev <- sort(nlev)
      } else {
        lev <- c(sort(nlev), max(x, na.rm=T)+1)
      }
    }
    cut(x, lev, labels=F)
  } else if (by == 4) {
    lev <- c(min(x, na.rm=T)-1, quantile(x, prob=nlev), max(x, na.rm=T)+1)
    cut(x, lev, labels=F)
  } else if (by == 5) {
    pts <- range(x)
    val <- pts[1] + diff(pts) * nlev
    fcategorize(x, val, by="fixed.lev")
  } else {
    stop("Invalid mode of separation")
  }
}


fcategorize2 <- function(x, nlev, by=c("equal.lev", "equal.quantile", "fixed.lev", "fixed.quantile", "fixed.range")) {
  by = pmatch(by, c("equal.lev", "equal.quantile", "fixed.lev", "fixed.quantile", "fixed.range"))
  if (by == 1) {
    lev <- seq(min(x, na.rm=T), max(x, na.rm=T), len=nlev+1)
    lev[which.min(lev)] <- min(x, na.rm=T)-1
    lev[which.max(lev)] <- max(x, na.rm=T)+1
    list(lev[2:nlev], cut(x, lev, labels=F))
  } else if (by == 2) {
    stp <- seq(0, 1, len=nlev+1)
    lev <- c(min(x, na.rm=T)-1, quantile(x, prob=stp[-c(1,nlev+1)]), max(x, na.rm=T)+1)
    list(lev[2:nlev], cut(x, lev, labels=F))
  } else if (by == 3) {
    m <- min(nlev); M <- max(nlev)
    if (any(x <= m, na.rm=T)) {
      if (all(x < M, na.rm=T)){
        lev <- c(min(x, na.rm=T)-1, sort(nlev))
      } else {
        lev <- c(min(x, na.rm=T)-1, sort(nlev), max(x, na.rm=T)+1)
      }
    } else {
      if (all(x < M, na.rm=T)) {
        lev <- sort(nlev)
      } else {
        lev <- c(sort(nlev), max(x, na.rm=T)+1)
      }
    }
    list(lev[2:(length(lev)-1)], cut(x, lev, labels=F))
  } else if (by == 4) {
    lev <- c(min(x, na.rm=T)-1, quantile(x, prob=nlev), max(x, na.rm=T)+1)
    list(lev[2:(length(lev)-1)], cut(x, lev, labels=F))
  } else if (by == 5) {
    pts <- range(x)
    val <- pts[1] + diff(pts) * nlev
    fcategorize2(x, val, by="fixed.lev")
  } else {
    stop("Invalid mode of separation")
  }
}


fcategorize.rev <- function(x) {
  rang <- range(x)
  base <- sum(rang)
  base - x
}


#########################################################################
## Basic functions
#########################################################################
geomean <- function(x) {
  prod(x)^(1/length(x))
}

fInvErf <- function(x) {
    return (qnorm((1+x)/2)/sqrt(2))
}

##############################################################################
## Transform distribution of input values into normal distribution.
##############################################################################
fNormalTransform <- function(x, avg, std) {
    pf <- ecdf(x)
    v <- get('x', envir=environment(pf))
    y <- get('y', envir=environment(pf))
    z <- avg + std * sqrt(2) * fInvErf(2 * y - 1)
    ord <- order(x)
    ord2 <- order(ord)
    invisible (z[ord2])
}

##############################################################################
## Transform distribution of input values into mean 'avg' and 
## standard deviation 'std'
##############################################################################
fStandardize <- function(x, avg, std) {
  if (length(which(is.na(x))) == 0) {
    (x - mean(x)) / sd(x) * sqrt(std) + avg
  } else {
    (x - mean(x, na.rm=T)) / sd(x, na.rm=T) * sqrt(std) + avg
  }
}


findex.mat.bare <- function(x, prb, val.type="in", alg="med") {
  com <- intersect(prb, rownames(x))
  if (length(com) == 0) stop("No probe for this signature.")
  vt <- pmatch(val.type, VALUE.TYPE)
  if (!(vt %in% 1:3)) stop("Invalid value type")
  algt <- pmatch(alg, ALG)
  if (!(algt %in% c(1,2))) stop("Invalid algorithm")
  return(apply(x[com,], 2, function(y) fsingle.bare.bare(y, vt, algt)))
}


findex.mat.bare.bare <- function(x, prb, val.type="in", alg="med") {
  findex.mat.bare(x, prb, val.type, alg)
}


fsingle.bare.bare <- function(x, vt, algt) {
  if (vt == 1) {  ## intensity type value
    invisible(switch(algt, median(x), geomean(x)))
  } else {  ## logratio type value
    invisible(switch(algt, median(x), mean(x)))
  }
}


findex.mat.bare.weight <- function(x, prb, val.type="in", alg="med"){
  if (ncol(prb) < 2) stop("Not enough probe information.")
  vt <- pmatch(val.type, VALUE.TYPE)
  if (!(vt %in% 1:3)) stop("Invalid value type")
  algt <- pmatch(alg, ALG)
  if (!(algt %in% c(1,2))) stop("Invalid algorithm")
  return(apply(x, 2, function(y) fsingle.bare.weight(y, prb, vt, algt)))
}


fsingle.bare.weight <- function(x, prb, vt, algt) {
  if (ncol(prb) < 2) stop("Not enough probe information.")
  com <- intersect(prb[[1]], names(x))
  if (length(com) == 0) stop("No probe for this signature.")
  df <- prb[match(com, prb[[1]]),]
  ups <- df[df[[2]] > 0,1]
  dns <- setdiff(df[[1]], ups)
  if (vt == 1) {  ## intensity type value
    if (length(ups) > 0 && length(dns) > 0) {
      res <- switch(algt, log2(median(x[ups]) / median(x[dns])),
                    log2(geomean(x[ups]) / geomean(x[dns])))
    } else if (length(ups) > 0) {
      res <- switch(algt, log2(median(x)), log2(geomean(x)))
    } else {
      res <- switch(algt, -log2(median(x)), -log2(geomean(x)))
    }
  } else {  ## logintensity or logratio type value
    if (length(ups) > 0 && length(dns) > 0) {
      res <- switch(algt, median(x[ups]) - median(x[dns]),
                    mean(x[ups]) - mean(x[dns]))
    } else if (length(ups) > 0) {
      res <- switch(algt, median(x), mean(x))
    } else {
      res <- switch(algt, -median(x), -mean(x))
    }
  }
  invisible(res)
}


findex.mat.norm.bare <- function(x, prb, val.type="in", alg="med") {
  com <- intersect(prb, rownames(x))
  if (length(com) == 0) stop("No probe for this signature.")
  vt <- pmatch(val.type, VALUE.TYPE)
  if (!(vt %in% 1:3)) stop("Invalid value type")
  algt <- pmatch(alg, ALG)
  if (!(algt %in% c(1,2))) stop("Invalid algorithm")
  vec <- apply(x[com,], 1, median)
  if (vt == 1) {
    gem <- log2(apply(x[com,], 2, function(y) y / vec))
  } else if (vt == 2) {
    gem <- apply(x[com,], 2, function(y) y - vec)
  } else {
    gem <- x[com,]
  }
  return(apply(gem, 2, function(y) fsingle.bare.bare(y, vt=2, algt=algt)))
}


fsingle.norm.bare <- function(x, vt, algt, med) {
  if (vt %in% c(1,2) && missing(med)) stop("Missing median")
  if (vt == 1) {  ## intensity type value
    v <- log2(x/med)
  } else if (vt == 2) {
    v <- x - med
  } else {  ## logratio type value
    v <- x
  }
  invisible(switch(algt,median(v),mean(v)))
}


findex.mat.norm.weight <- function(x, prb, val.type="in", alg="med") {
  if (ncol(prb) < 2) stop("Not enough probe information.")
  vt <- pmatch(val.type, VALUE.TYPE)
  if (!(vt %in% 1:3)) stop("Invalid value type")
  algt <- pmatch(alg, ALG)
  if (!(algt %in% c(1,2))) stop("Invalid algorithm")
  if (vt == 1) {
    vec <- apply(x, 1, median)
    gem <- log2(apply(x, 2, function(y) y / vec))
  } else if (vt == 2) {
    vec <- apply(x, 1, median)
    gem <- apply(x, 2, function(y) y - vec)
  } else {
    gem <- x
  }
  return(apply(gem, 2, function(y) fsingle.bare.weight(y, prb=prb, vt=2, algt=algt)))
}


fsingle.norm.weight <- function(x, prb, vt, algt, med) {
  if (vt %in% c(1,2) && missing(med)) stop("Missing median")
  if (ncol(prb) < 2) stop("Not enough probe information.")
  com <- intersect(prb[[1]], names(x))
  if (length(com) == 0) stop("No probe for this signature.")
  prb <- prb[match(com, prb[[1]]),]
  ups <- prb[prb[[2]] > 0, 1]
  dns <- setdiff(prb[[1]], ups)
  if (vt == 1) {  ## intensity type value
    v <- log2(x/med)
  } else if (vt == 2) {  ## logintensity type value
    v <- x - med
  } else {  ## logratio type value
    v <- x
  }
  if (length(ups) > 0 && length(dns) > 0) {
    res <- switch(algt, median(v[ups]) - median(v[dns]),
                  mean(v[ups]) - mean(v[dns]))
  } else if (length(ups) > 0) {
    res <- switch(algt, median(v[ups]), mean(v[ups]))
  } else {
    res <- switch(algt, -median(v[dns]), -mean(v[dns]))
  }
  invisible(res)
}


findex.mat <- function(x, prb, val.type="in", alg="med", normalize=T, weight=T) {
  if (normalize) {
    if (weight) {
      findex.mat.norm.weight(x, prb, val.type, alg)
    } else {
      findex.mat.norm.bare(x, prb, val.type, alg)
    }
  } else {
    if (weight) {
      findex.mat.bare.weight(x, prb, val.type, alg)
    } else {
      findex.mat.bare.bare(x, prb, val.type, alg)
    }
  }
}


###############################################################################
## Specific functions
###############################################################################

##=============================================================================
##   This function calculates proliferation index (PI) from a MAS5 transformed
##   intensity data in a repository. This function calculates PI in 2 steps:
##   first, it normalize each probeset profile by its median and obtain
##   log2 (probeset-wise-median-normalized-signal) values; second, mean of
##   logratio values over marker probesets for each sample. Mean value is
##   chosen since PI from intensity data is calculated by goemetric mean.
## \usage{ fProlifIndex(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
##  \item{val.type}{value type ("intensity", "logintensity", "logratio")}
## }
##=============================================================================
fProlifIndex <- function(x, val.type="in") {
  findex.mat.norm.bare(x, PROBE$PI$PROBE, val.type, "med")
}


fPI <- function(x, val.type="in") {
  fProlifIndex(x, val.type=val.type)
}


##=============================================================================
##   This function calculates proliferation index (PI) from a MAS5 transformed
##   intensity vector. This function calculates PI in 2 steps:
##   first, it normalize each probeset profile by its median and obtain
##   log2 (probeset-wise-median-normalized-signal) values; second, median
##   or mean of logratio values over marker probesets for each sample.
##   Mean value is chosen since PI from intensity data is calculated by
##   goemetric mean.
## \usage{ fSingleProlifIndex(x, med, alg=c("median", "geomean")) }
## \arguments{
##  \item{x}{vector of real (expression values)}
##  \item{alg}{vector of real (reference expression values for normalization)}
## }
##=============================================================================
fSingleProlifIndex <- function(x, med, val.type="in") {
  com <- intersect(names(x), PROBE$PI$PROBE)
  if (length(com) == 0) stop("No probe for this signature.")
  fsingle.norm.bare(x[com], pmatch(val.type, VALUE.TYPE), 1, med[com])
}


fSinglePI <- function(x, med, val.type="in") {
  fSingleProlifIndex(x, med, val.type=val.type)
}


##=============================================================================
##   This function calculates centrosome index (CI) from a MAS5 transformed
##   intensity dataset in a repository.
## \usage{ fCentroIndex(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
## }
##=============================================================================
fCentroIndex <- function(x, val.type="in") {
  vt <- pmatch(val.type, VALUE.TYPE)
  if (!(vt %in% 1:3)) stop("Invalid value type")
  prb1 <- intersect(c("209194_at", "201714_at"), rownames(x))
  prb2 <- intersect(c("218014_at", "203660_s_at", "233387_s_at"), rownames(x))
  com <- c(prb1, prb2)
  if (length(com) == 0) stop("No probe for this signature.")
  df <- x[com,,drop=F]
  if (vt == 1) {  ## intensity type
    med <- apply(df, 1, median)
    df <- apply(df, 2, function(y) log2(y / med))
  } else if (vt == 2) {  ## log-intensity type
    med <- apply(df, 1, median)
    df <- apply(df, 2, function(y) y - med)
  } ## no processing for log-ratio type
  if (length(prb1) > 0 & length(prb2) > 0) {
    res <- apply(df, 2, function(x) sum(c(x[prb1], median(x[prb2]))))
  } else {
    if (length(prb1) > 0) {
      res <- apply(df, 2, sum)
    } else {
      res <- apply(df, 2, median)
    }
  }
  invisible(res)
}


fCI <- function(x, val.type="in") {
  fCentroIndex(x, val.type)
}


##=============================================================================
##   This function calculates centrosome index (CI) from a MAS5 transformed
##   intensity vector.
## \usage{ fSingleCentroIndex(x, med) }
## \arguments{
##  \item{x}{vector of real (expression values)}
##  \item{med}{vector of real (reference expression values)}
## }
##=============================================================================
fSingleCentroIndex <- function(x, med, val.type="in") {
  vt <- pmatch(val.type, VALUE.TYPE)
  if (!(vt %in% 1:3)) stop("Invalid value type")
  ## prb <- c("209194_at", "201714_at", "218014_at", "203660_s_at", "233387_s_at")
  prb1 <- c("209194_at", "201714_at")
  prb2 <- c("218014_at", "203660_s_at", "233387_s_at")
  com1 <- intersect(prb1, names(x))
  com2 <- intersect(prb2, names(x))
  com <- c(com1, com2)
  if (length(com) == 0) stop("No probe for this signature.")
  if (vt %in% c(1,2) && missing(med)) stop("Missing median")
  if (vt == 1) { ## intensity type
    if (length(com1) > 0 & length(com2) > 0) {
      res <- log2(x[com]/med[com])
      res <- sum(c(res[com1], median(res[com2])))
    } else if (length(com1) > 0) {
      res <- log2(x[com1]/med[com1])
      res <- sum(res[com1])
    } else {
      res <- log2(x[com2]/med[com2])
      res <- median(res[com2])
    }
  } else if (vt == 2) {  ## log-intensity type
    if (length(com1) > 0 & length(com2) > 0) {
      res <- x[com] - med[com]
      res <- sum(c(res[com1], median(res[com2])))
    } else if (length(com1) > 0) {
      res <- x[com1] - med[com1]
      res <- sum(res[com1])
    } else {
      res <- x[com2] - med[com2]
      res <- median(res[com2])
    }
  } else { ## log-ratio type
    if (length(com1) > 0 & length(com2) > 0) {
      res <- sum(c(x[com1], median(x[com2])))
    } else if (length(com1) > 0) {
      res <- sum(x[com1])
    } else {
      res <- median(x[com2])
    }
  }
  invisible(res)
}


fSingleCI <- function(x, med, val.type="in") {
  fSingleCentroIndex(x, med, val.type)
}


##=============================================================================
##   This function calculates prognostic index based on UAMS 70-gene signature.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fUAMS70(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
## }
##=============================================================================
fUAMS70 <- function(x, val.type="in") {
  findex.mat.norm.weight(x, PROBE$UAMS70, val.type, "median")
}


##=============================================================================
##   This function calculates prognostic index based on UAMS 70-gene signature.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fSingleUAMS70(x, med, val.type="in") }
## \arguments{
##  \item{x}{vector of real (expression values)}
## }
##=============================================================================
fSingleUAMS70 <- function(x, med, val.type="in") {
  fsingle.norm.weight(x, PROBE$UAMS70, pmatch(val.type, VALUE.TYPE), 2, med)
}


##=============================================================================
##   This function calculates prognostic index based on UAMS 70-gene signature.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fUAMS80(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
## }
##=============================================================================
fUAMS80 <- function(x, val.type="in") {
  findex.mat.norm.weight(x, PROBE$UAMS80, val.type, "median")
}


##=============================================================================
##   This function calculates prognostic index based on UAMS 70-gene signature.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fSingleUAMS80(x, med, val.type="in") }
## \arguments{
##  \item{x}{vector of real (expression values)}
## }
##=============================================================================
fSingleUAMS80 <- function(x, med, val.type="in") {
  fsingle.norm.weight(x, PROBE$UAMS80, pmatch(val.type, VALUE.TYPE), 2, med)
}


##=============================================================================
##   This function calculates prognostic index based on EMC 92-gene signature.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fEMC92(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
## }
##=============================================================================
fEMC92 <- function(x, val.type="in") {
  findex.mat.norm.weight(x, PROBE$EMC92, val.type, "median")
}


##=============================================================================
##   This function calculates prognostic index based on UAMS 70-gene signature.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fSingleEMC92(x, med, val.type="in") }
## \arguments{
##  \item{x}{vector of real (expression values)}
## }
##=============================================================================
fSingleEMC92 <- function(x, med, val.type="in") {
  fsingle.norm.weight(x, PROBE$EMC92, pmatch(val.type, VALUE.TYPE), 1, med)
}


##=============================================================================
##   This function calculates prognostic index based on IFM 15-gene signature.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fIFM15(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
## }
##=============================================================================
fIFM15 <- function(x, val.type="in") {
  findex.mat.norm.weight(x, PROBE$IFM15, val.type)
}


##=============================================================================
##   This function calculates prognostic index based on IFM 15-gene signature.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fSingleIFM15(x, med, val.type="in") }
## \arguments{
##  \item{x}{vector of real (expression values)}
## }
##=============================================================================
fSingleIFM15 <- function(x, med, val.type="in") {
  fsingle.norm.weight(x, PROBE$IFM15, pmatch(val.type, VALUE.TYPE), 1, med)
}


##=============================================================================
##   This function calculates prognostic index based on MM cell line derived
##   7-probe signature. The index is calcualted by the sum of
##   MAS5-processed expression values.
## \usage{ fHMCL7(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
## }
##=============================================================================
fHMCL7 <- function(x, val.type="in") {
  findex.mat.norm.bare(x, PROBE$HMCL7$PROBE, val.type, "med")
}


##=============================================================================
##   This function calculates prognostic index based on MM cell line derived
##   7-probe signature. The index is calcualted by the sum of
##   MAS5-processed expression values.
## \usage{ fSingleHMCL7(x, med, val.type="in") }
## \arguments{
##  \item{x}{vector of real (expression values)}
## }
##=============================================================================
fSingleHMCL7 <- function(x, med, val.type="in") {
  com <- intersect(names(x), PROBE$HMCL7$PROBE)
  if (length(com) == 0) stop("No probe for this signature.")
  fsingle.norm.bare(x[com], pmatch(val.type, VALUE.TYPE), 1, med[com])
}


##=============================================================================
##   This function calculates prognostic index based on cell death genes in
##   homozygously deleted regions.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fHZDCD(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
## }
##=============================================================================
fHZDCD <- function(x, val.type="in") {
  findex.mat.norm.weight(x, PROBE$HZDCD, val.type)
}


##=============================================================================
##   This function calculates prognostic index based on cell death genes in
##   homozygously deleted regions.
##   The index is calcualted by log2 (average_up / average_down) using
##   MAS5-processed expression values.
## \usage{ fSingleHZDCD(x, med, val.type="in") }
## \arguments{
##  \item{x}{vector of real (expression values)}
## }
##=============================================================================
fSingleHZDCD <- function(x, med, val.type="in") {
  fsingle.norm.weight(x, PROBE$HZDCD, pmatch(val.type, VALUE.TYPE), 1, med)
}


##=============================================================================
##   This function calculates stratification index based on MM cell line
##   derived 248-probe signature. The index is calcualted by the average of
##   median-normalized MAS5-processed expression values.
## \usage{ fHMCL248(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
## }
##=============================================================================
fHMCL248 <- function(x, val.type="in") {
  findex.mat.norm.bare(x, PROBE$HMCL248$PROBE, val.type, "med")
}


##=============================================================================
##   This function calculates prognostic index based on MM cell line derived
##   248-probe signature. The index is calcualted by the average of
##   median-normalized MAS5-processed expression values.
## \usage{ fSingleHMCL248(x, med, val.type="in") }
## \arguments{
##  \item{x}{vector of real (expression values)}
## }
##=============================================================================
fSingleHMCL248 <- function(x, med, val.type="in") {
  com <- intersect(names(x), PROBE$HMCL248$PROBE)
  if (length(com) == 0) stop("No probe for this signature.")
  fsingle.norm.bare(x[com], pmatch(val.type, VALUE.TYPE), 1, med[com])
}


##=============================================================================
##   This function calculates survival index based on chromosome instability (CIN)
##   215-probe signature. The index is calcualted by log2 (median_up / median_down)
##   using MAS5-processed expression values.
## \usage{ fCINGEC(x, val.type="in") }
## \arguments{
##  \item{x}{gene expression matrix}
##=============================================================================
fCINGEC <- function(x, val.type="in") {
  findex.mat.norm.weight(x, PROBE$CINGEC, val.type, "median")
}


##=============================================================================
##   This function calculates prognostic index based on UAMS 70-gene signature.
##   The index is calcualted by median_up - median_down using median-normalized
##   MAS5-processed expression values.
## \usage{ fSingleCINGEC(x, med, val.type="in") }
## \arguments{
##  \item{x}{vector of real (expression values)}
##  \item{med}{vector of real (median expression values)}
## }
##=============================================================================
fSingleCINGEC <- function(x, med, val.type="in") {
  fsingle.norm.weight(x, PROBE$CINGEC, pmatch(val.type, VALUE.TYPE), 1, med)
}


##=============================================================================
##   This function calculates prognostic indices over a pack of signatures
##   provided by input parameter \code{pack}.
## }
## \arguments{
##  \item{x}{matrix of real (median normalized gene expression matrix)}
##  \item{pack}{vector of character (signature names)}
##  \item{FILE}{vector of character (named vector for signature files
##              corresponding to signatures whose names are in pack but not in
##              \code{SIGPACK})}
## }
##=============================================================================
fGeneSigPack <- function(x, val.type="in") {
  vt <- pmatch(val.type, VALUE.TYPE)
  if (vt == 1) {  ## intensity type
    med <- apply(x, 1, median)
    gep <- log2(apply(x, 2, function(y) y / med))
  } else if (vt == 2) {  ## log-intensity type
    med <- apply(x, 1, median)
    gep <- apply(x, 2, function(y) y - med)
  } else if (vt == 3) {  ## logratio type
    gep <- x
  } else {
    stop("Invalid value type")
  }
  res <- fCINGEC(gep, "logratio")
  print ("...... CINGEC index ... done")
  res <- rbind(res, fCI(gep, "logratio"))
  print ("...... CNTI index ... done")
  res <- rbind(res, fEMC92(gep, "logratio"))
  print ("...... EMC92 index ... done")
  res <- rbind(res, fHMCL7(gep, "logratio"))
  print ("...... HMCL7 index ... done")
  res <- rbind(res, fHZDCD(gep, "logratio"))
  print ("...... HZDCD index ... done")
  res <- rbind(res, fIFM15(gep, "logratio"))
  print ("...... IFM15 index ... done")
  res <- rbind(res, fPI(gep, "logratio"))
  print ("...... Proliferation index ... done")
  res <- rbind(res, fUAMS70(gep, "logratio"))
  print ("...... UAMS70 index ... done")
  res <- rbind(res, fUAMS80(gep, "logratio"))
  print ("...... UAMS80 index ... done")
  rownames(res) <- c("CINGEC", "CNTI", "EMC92", "HMCL7", "HZDCD", "IFM15", "PI", "UAMS70", "UAMS80")
  invisible(res)
}

SIGPACK <- c("CINGEC", "CNTI", "EMC92", "HMCL7", "HZDCD", "IFM15", "PI", "UAMS70", "UAMS80")

fGeneSigPack2 <- function(x, pack, val.type="in") {
  vt <- pmatch(val.type, VALUE.TYPE)
  if (vt == 1) {  ## intensity type
    med <- apply(x, 1, median)
    gep <- log2(apply(x, 2, function(y) y / med))
  } else if (vt == 2) {  ## log-intensity type
    med <- apply(x, 1, median)
    gep <- apply(x, 2, function(y) y - med)
  } else if (vt == 3) {  ## logratio type
    gep <- x
  } else {
    stop("Invalid value type")
  }
  res <- NULL
  for (id in names(pack)) {
    if (id == "CNTI") {
      res <- rbind(res, fCI(gep, "logratio"))
    } else {
      sig <- pack[[id]]
      
      if (length(sig) == 1 || length(unique(sig[[2]])) == 1) {
        res <- rbind(res, findex.mat(gep, sig[[1]], val.type="logratio", normalize=F, weight=F))
      } else {
        if (id == "UAMS70") {
          res <- rbind(res, findex.mat(gep, sig, val.type="logratio", normalize=F, alg="mean"))
        } else {
          res <- rbind(res, findex.mat(gep, sig, val.type="logratio", normalize=F))
        }
      }
    }
    print (paste("......", id, "index ... done"))
  }
  rownames(res) <- names(pack)
  invisible(res)
}


##
## \name{fBox}
## \alias{fBox}
## \title{modified boxplot}
## \description{
##  This is a slightly modified version of default boxplot.
## }
## \usage{
##  fbox(dat, xlab, ylab, main, ...)
## }
## \arguments{
##  \item{dat}{list.}
##  \item{xlab, ylab}{x- and y-axis labels.}
##  \item{main}{main title.}
##  \item{...}{miscellaneous plot options.}
## }
## \details{ }
## \values{ }
## \examples{ }
## \author{Tae-Hoon Chung (hoontaechung@gmail.com)}
## \keyword{plot}
##
fBox <- function(x, ...) {
  cols <- brewer.pal(length(x), "Set1") ## from RColorBrewer package
  boxplot(x, outline=F, ...)
  ns <- sapply(x, length)
  for (n in 1:length(x)) {
    points(n+runif(ns[n], -0.2, 0.2), x[[n]], pch=21, bg=cols[n], lwd=0.25, ...)
  }
}


## hacked from bioconductor project.
## --- function to plot color bar.
fColorBar <- function (x, horizontal = TRUE, col = heat.colors(50),
                      scale = 1:length(x), k = 7, CEX=1, ...) {
  if (is.numeric(x)) {
    x <- x
    colmap <- col
  } else {
    colmap <- x
    low <- range(scale)[1]
    high <- range(scale)[2]
    x <- seq(low, high, length = length(x))
  }
  if (length(x) > k) x.small <- seq(x[1], x[length(x)], length = k)
  else x.small <- x
  if (horizontal) {
    image(x, 1, matrix(x, length(x), 1), axes = FALSE, xlab = "", ylab = "",
          col = colmap, ...)
    ##par(las=2)
    mtext(as.character(signif(rev(x.small),2)), side=1, adj=0.5, #adj=1.3,
          at=rev(x.small), cex=CEX)#, srt=270)
    ##axis(1, at=rev(x.small), labels=signif(rev(x.small),2), tck=0,
    ##     cex.axis=CEX)
    par(las=0)
  }
  if (!horizontal) {
    image(1, x, matrix(x, 1, length(x)), axes = FALSE, xlab = "", ylab = "",
          col = colmap, ...)
    ##par(las = 1)
    mtext(as.character(signif(rev(x.small), 2)), side=4, adj=0.25, line=1,
          at=rev(x.small), cex=CEX, srt=90)
    ##axis(4, at = rev(x.small), labels = signif(rev(x.small), 2), tck=0,
    ##     cex.axis=CEX)
    par(las = 0)
  }
  box(...)
}


## generate colors for pre-specified mode.
fColorScheme <- function(mode) {
	## --- hacked from bioconductor project.
	## --- function to retrieve R, G, B color codes from string specification.
	GetRGB <- function (low="white",high=c("green","red"),mid=NULL,k=50) {
	  low <- col2rgb(low)/255; high <- col2rgb(high)/255
	  if (is.null(mid)) {
		r <- seq(low[1],high[1],len=k)
		g <- seq(low[2],high[2],len=k)
		b <- seq(low[3],high[3],len=k)
	  }
	  if (!is.null(mid)) {
		k2 <- round(k/2); mid <- col2rgb(mid)/255
		r <- c(seq(low[1],mid[1],len=k2), seq(mid[1],high[1],len=k2))
		g <- c(seq(low[2],mid[2],len=k2), seq(mid[2],high[2],len=k2))
		b <- c(seq(low[3],mid[3],len=k2), seq(mid[3],high[3],len=k2))
	  }
	  return(rgb(r, g, b))
	}

  if (length(mode) == 1) {
    if (mode == 1) {
        colM <- GetRGB(low="green", mid="black", high="red", k=50)
    } else if (mode == 2) {
        colM <- GetRGB(low="green", mid="white", high="red", k=50)
    } else if (mode == 3) {
        colM <- GetRGB(low="red", mid="black", high="green", k=50)
    } else if (mode == 4) {
        colM <- GetRGB(low="red", mid="white", high="green", k=50)
    } else if (mode == 5) {
        colM <- GetRGB(low="white", high="red", k=25)
    } else if (mode == 6) {
        colM <- GetRGB(low="white", high="green", k=25)
    } else if (mode == 7) {
        colM <- GetRGB(low="black", high="red", k=25)
    } else if (mode == 8) {
        colM <- GetRGB(low="black", high="green", k=25)
    } else if (mode == 9) {
        colM <- GetRGB(low="black", high="white", k=25)
    } else if (mode == 10) {
        colM <- GetRGB(low="white", high="black", k=25)
    } else {
      stop("Wrong color scheme")
    }
  } else {
    if (length(mode) == 2) {
      colM <- GetRGB(low=mode[1], high=mode[2], k=25)
    } else if (length(mode) == 3) {
      colM <- GetRGB(low=mode[1], mid=mode[2], high=mode[3], k=50)
    } else {
      stop("Wrong color scheme")
    }
  }
  return (colM)
}

##
## \name{fHmap}
## \alias{fHmap}
## \title{bare heatmap generation.}
## \description{
##  This is a function to generate bare heatmap from a matrix.
## }
## \usage{
##  fHmap(z, Range=NULL, Grid=TRUE, Color=1,
##        GridLine=list(Type=1, Col="orange", Weight=1),
##        RowLine=list(Pos=NULL, Type=1, Weight=1, Col="cyan"),
##        ColLine=list(Pos=NULL, Type=1, Weight=1, Col="cyan"))
## }
## \arguments{
##  \item{z}{matrix}
##  \item{Range}{range of z values to plot.}
##  \item{Grid}{show grid lines?}
##  \item{GridLine}{grid line specification ("Col" for line color,
##    "Weight" for line weight, "Type" for line type).}
##  \item{RowLine}{horizontal line specification ("Pos" for line position,
##    "Type" for line type, "Weight" for line weight, "Col" for line color).}
##  \item{ColLine}{vertical line specification ("Pos" for line position,
##    "Type" for line type, "Weight" for line weight, "Col" for line color).}
##  \item{Color}{heatmap color specification (see details)}
##  \item{Mark}{list of positions & border line specs for marking.}
## }
## \details{
##   items for Mark should look like this:
##     Mark=list(list(R=c(1,1), C=c(3,5), Type=1, Weight=1, Col="red"),
##               list(R=c(2,3), C=c(1,6), Type=1, Weight=1, Col="blue"))
## }
## \values{ }
## \examples{ }
## \author{Tae-Hoon Chung (hoontaechung@gmail.com)}
## \keyword{plot}
##
fHmap <- function(z, Range=NULL, Grid=TRUE, Color=1,
                  GridLine=list(Col="orange", Weight=1, GridType=1),
                  RowLine=list(Pos=NULL, Type=1, Weight=1, Col="cyan"),
                  ColLine=list(Pos=NULL, Type=1, Weight=1, Col="cyan"),
                  Mark=NULL) {
  def.par <- par(no.readonly=T)

  colM <- fColorScheme(Color)

  y <- t(z[nrow(z):1,])

  if (!is.null(Range)) {
    ind <- which(y >= Range[2]); y[ind] <- Range[2]
    ind <- which(y <= Range[1]); y[ind] <- Range[1]
  } else Range <- range(z, na.rm=T)

  par(mar=rep(0,4))

  image(0:ncol(z), 0:nrow(z), y, col=colM, axes=FALSE, zlim=Range)
  box(lwd=GridLine$Weight*1.1)

  axis.hor <- nrow(z)-1; axis.ver <- ncol(z)-1

  if (Grid) {
    abline(v=1:(ncol(z)-1),
           col=GridLine$Col, lty=GridLine$Type, lwd=GridLine$Weight)
    abline(h=1:(nrow(z)-1),
           col=GridLine$Col, lty=GridLine$Type, lwd=GridLine$Weight)
    box()
  }

  if (!is.null(RowLine$Pos))
    abline(h=nrow(z)-RowLine$Pos,
           lty=RowLine$Type, lwd=RowLine$Weight, col=RowLine$Col)
  if (!is.null(ColLine$Pos))
    abline(v=ColLine$Pos,
           lty=ColLine$Type, lwd=ColLine$Weight, col=ColLine$Col)

  if (!is.null(Mark)) {
    for(item in Mark) {
      segments(item$C-1, nrow(z)-item$R, item$C, nrow(z)-item$R,
               col=item$Col, lty=item$Type, lwd=item$Weight)
      segments(item$C, nrow(z)-item$R, item$C, nrow(z)-item$R+1,
               col=item$Col, lty=item$Type, lwd=item$Weight)
      segments(item$C-1, nrow(z)-item$R+1, item$C, nrow(z)-item$R+1,
               col=item$Col, lty=item$Type, lwd=item$Weight)
      segments(item$C-1, nrow(z)-item$R, item$C-1, nrow(z)-item$R+1,
               col=item$Col, lty=item$Type, lwd=item$Weight)
    }
  }
  
  par <- def.par

  invisible (colM)
}


## hacked from PerformanceAnalytics library with slight modification.
## --- function to generate pairs plot.
fPairs <- function (R, histogram = TRUE,
                    method = c("pearson", "kendall", "spearman"), CEXCOR=1,
                    fit="lin", ...) {
  x <- checkData(R, method = "matrix")
  if (missing(method)) method <- "pearson"

  panel.cor <- function(x, y, digits = 2, prefix = "",
                        use = "pairwise.complete.obs", ...) {
    usr <- par("usr")
    on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- cor(x, y, use = use, method = method)
    txt <- format(c(r, 0.123456789), digits = digits)[1]
    txt <- paste(prefix, txt, sep = "")
    test <- cor.test(x, y)
    Signif <- symnum(test$p.value, corr = FALSE, na = FALSE,
                     cutpoints = c(0, 0.001, 0.01, 0.05, 0.1, 1),
                     symbols = c("***", "**", "*", ".", " "))
    ## text(0.5, 0.5, txt, cex = cex * r)
    text(0.5, 0.5, txt, cex=CEXCOR, adj=0.5)
    text(0.8, 0.8, Signif, col = 2)
  }
  hist.panel <- function(x, ...) {
    par(new = TRUE)
    d <- density(x, na.rm = TRUE)
    h <- hist(x, breaks = "FD", plot=F)
    M <- max(d$y, h$density)
    plot(h, col = "light gray", freq=F, axes=FALSE, main="", ylim=c(0, M))
    lines(d, col = "red", lwd = 1)
    rug(x)
  }
  lin.panel <- function(x, y, ...) {
      mat <- cbind(x, y)
      points(mat, col=densCols(mat), ...)
      if (fit == "lin") abline(lm(y ~ x), col=2)
      else lines(lowess(x, y, f=1/10), col=2)
  }
  if (histogram) {
    pairs(x, gap = 0, lower.panel = lin.panel, upper.panel = panel.cor,
          diag.panel = hist.panel, ...)
  } else {
    pairs(x, gap = 0, lower.panel = panel.smooth, upper.panel = panel.cor,
          ...)
  }
}


## this one hacked from "Cookbook for R" site
##
## Multiple plot function
##
## ggplot objects can be passed in ..., or to plotlist (as a list of ggplot objects)
## - cols:   Number of columns in layout
## - layout: A matrix specifying the layout. If present, 'cols' is ignored.
##
## If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
## then plot 1 will go in the upper left, 2 will go in the upper right, and
## 3 will go all the way across the bottom.
##
fMultiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  ## require(grid)

  ## Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)

  numPlots = length(plots)

  ## If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    ## Make the panel
    ## ncol: Number of columns of plots
    ## nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols), byrow=T)
  }

  if (numPlots==1) {
    print(plots[[1]])

  } else {
    ## Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))

    ## Make each plot, in the correct location
    for (i in 1:numPlots) {
      ## Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))

      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                          layout.pos.col = matchidx$col))
    }
  }
}
##======================= End of MMGeneSigIndex.R ========================
